@@ -18,6 +18,8 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 #include "svnfrontend/createrepo_impl.h"
+#include "src/svnqt/version_check.hpp"
+#include "src/svnqt/repoparameter.hpp"
 
 #include <kdebug.h>
 #include <kurl.h>
@@ -34,25 +36,49 @@ public:
     ~RecurseCheck(){value = false;}
 };
 
-Createrepo_impl::Createrepo_impl(bool enable_compat13, bool enable_compat14, QWidget *parent, const char *name)
-    :CreateRepo_Dlg(parent, name)
+struct CreateRepoData
 {
-    inChangeCompat=true;
+    bool inChangeCompat;
+    mutable svn::repository::CreateRepoParameter params;
+};
+
+Createrepo_impl::Createrepo_impl(QWidget *parent, const char *name)
+//     :CreateRepo_Dlg(parent, name)
+    : QWidget(parent)
+{
+    setupUi(this);
+    if (name) {
+        setObjectName(name);
+    } else {
+        setObjectName("Createrepo_impl");
+    }
+    m_ReposPathinput->setMode(KFile::Directory|KFile::LocalOnly);
+    _data = new CreateRepoData;
+
+   _data->inChangeCompat=true;
     m_DisableFsync->setEnabled(false);
     m_LogKeep->setEnabled(false);
-    if (!enable_compat13){
+
+    if ( svn::Version::version_major()>1||svn::Version::version_minor()>3 ){
+        m_svn13compat->setEnabled(true);
+    } else {
         m_svn13compat->setEnabled(false);
         m_svn13compat->hide();
-    } else {
-        m_svn13compat->setEnabled(true);
     }
-    if (!enable_compat14){
+    if (svn::Version::version_major()>1||svn::Version::version_minor()>4){
+        m_svn14compat->setEnabled(true);
+    } else {
         m_svn14compat->setEnabled(false);
         m_svn14compat->hide();
-    } else {
-        m_svn14compat->setEnabled(true);
     }
-    inChangeCompat=false;
+    if (svn::Version::version_major()>1||svn::Version::version_minor()>5){
+        m_svn15compat->setEnabled(true);
+    } else {
+        m_svn15compat->setEnabled(false);
+        m_svn15compat->hide();
+    }
+
+   _data->inChangeCompat=false;
 }
 
 void Createrepo_impl::fsTypeChanged(int which)
@@ -61,66 +87,72 @@ void Createrepo_impl::fsTypeChanged(int which)
     m_LogKeep->setEnabled(which==1);
 }
 
-QString Createrepo_impl::targetDir()
+QString Createrepo_impl::targetDir()const
 {
-    KURL u = m_ReposPathinput->url();
+    KUrl u = m_ReposPathinput->url();
     QString res = u.path();
-    while (res.endsWith("/")) {
+    while (res.endsWith('/')) {
         res.truncate(res.length()-1);
     }
     return res;
 }
 
-QString Createrepo_impl::fsType()
-{
-    return m_FilesystemSelector->currentText();
-}
-
-bool Createrepo_impl::disableFsync()
-{
-    return m_DisableFsync->isChecked();
-}
-
-bool Createrepo_impl::keepLogs()
-{
-    return m_LogKeep->isChecked();
-}
-
-bool Createrepo_impl::createMain()
+bool Createrepo_impl::createMain()const
 {
     return m_CreateMainDirs->isChecked();
 }
 
-bool Createrepo_impl::compat13()const
+void Createrepo_impl::compatChanged15(bool)
 {
-    return (m_svn13compat->isChecked()||!m_svn13compat->isEnabled());
-}
-
-bool Createrepo_impl::compat14()const
-{
-    return (m_svn14compat->isChecked()||!m_svn14compat->isEnabled());
+    if ((_data->inChangeCompat)) {
+        return;
+    }
+    RecurseCheck rc((_data->inChangeCompat));
+    if (m_svn15compat->isChecked()) {
+        m_svn13compat->setChecked(false);
+        m_svn14compat->setChecked(false);
+    }
 }
 
 void Createrepo_impl::compatChanged14(bool)
 {
-    if (inChangeCompat) {
+    if ((_data->inChangeCompat)) {
         return;
     }
-    RecurseCheck rc(inChangeCompat);
+    RecurseCheck rc((_data->inChangeCompat));
     if (m_svn14compat->isChecked()) {
+        if (m_svn15compat->isEnabled()) {
+            m_svn15compat->setChecked(false);
+        }
         m_svn13compat->setChecked(false);
     }
 }
 
 void Createrepo_impl::compatChanged13(bool)
 {
-    if (inChangeCompat) {
+    if ((_data->inChangeCompat)) {
         return;
     }
-    RecurseCheck rc(inChangeCompat);
-    if (m_svn13compat->isChecked() && m_svn14compat->isEnabled()) {
-        m_svn14compat->setChecked(false);
+    RecurseCheck rc((_data->inChangeCompat));
+    if (m_svn13compat->isChecked()) {
+        if (m_svn14compat->isEnabled()) {
+            m_svn14compat->setChecked(false);
+        }
+        if (m_svn15compat->isEnabled()) {
+            m_svn15compat->setChecked(false);
+        }
     }
 }
 
+const svn::repository::CreateRepoParameter&Createrepo_impl::parameter()const
+{
+    _data->params.path(targetDir());
+    _data->params.pre14_compat((m_svn13compat->isChecked()||!m_svn13compat->isEnabled()));
+    _data->params.pre15_compat((m_svn14compat->isChecked()||!m_svn14compat->isEnabled()));
+    _data->params.fstype(m_FilesystemSelector->currentText());
+    _data->params.bdbnosync(m_DisableFsync->isChecked());
+    _data->params.bdbautologremove(!m_LogKeep->isChecked());
+    return _data->params;
+}
+
 #include "createrepo_impl.moc"
