@@ -1,3 +1,8 @@
+/*
+ * Port for usage with qt-framework and development for kdesvn
+ * (C) 2005-2007 by Rajko Albrecht (ral@alwins-world.de)
+ * http://kdesvn.alwins-world.de
+ */
 /*
  * ====================================================================
  * Copyright (c) 2002-2005 The RapidSvn Group.  All rights reserved.
@@ -26,18 +31,21 @@
 #ifndef _SVNCPP_CLIENT_IMPL_H_
 #define _SVNCPP_CLIENT_IMPL_H_
 
-#include "client.hpp"
-#include "svnqt_defines.hpp"
+#include "svnqt/client.hpp"
+#include "svnqt/svnqt_defines.hpp"
 
 // Ignore MSVC 6 compiler warning: debug symbol truncated
 #if defined (_MSC_VER) && _MSC_VER <= 1200
 #pragma warning (disable: 4786)
 #endif
 
-// Ignore MSVC 7 compiler warning: C++ exception specification
-#if defined (_MSC_VER) && _MSC_VER > 1200 && _MSC_VER <= 1310
+// Ignore MSVC 7, 2005 & 2008 compiler warning: C++ exception specification
+#if defined (_MSC_VER) && _MSC_VER > 1200 && _MSC_VER <= 1550
 #pragma warning (disable: 4290)
 #endif
+
+class QStringList;
+
 namespace svn
 {
   namespace stream {
@@ -47,20 +55,20 @@ namespace svn
   /**
    * Subversion client API.
    */
-  SVNQT_NOEXPORT class Client_impl:public Client
+  class SVNQT_NOEXPORT Client_impl:public Client
   {
   public:
     /**
      * Initializes the primary memory pool.
      */
-    Client_impl(Context * context = 0);
+    Client_impl(const ContextP&context);
 
     virtual ~Client_impl();
 
     /**
      * @return returns the Client context
      */
-    virtual const Context *
+    virtual const ContextP
     getContext () const;
 
     /**
@@ -71,7 +79,7 @@ namespace svn
      * @param context new context to use
      */
     virtual void
-    setContext (Context * context = NULL);
+    setContext (const ContextP&context);
 
 
     /**
@@ -79,26 +87,11 @@ namespace svn
      *
      * Throws an exception if an error occurs
      *
-     * @param path Path to explore.
-     * @param descend Recurse into subdirectories if existant.
-     * @param get_all Return all entries, not just the interesting ones.
-     * @param update Query the repository for updates.
-     * @param no_ignore Disregard default and svn:ignore property ignores.
-     * @param hide_externals don't recurse into external definitions
-     * @param revision list specific revision when browsing remote, on working copies parameter will ignored
-     * @param detailed_remote if on remote listing detailed item info should get if possible
-     *                        that may slow so should configureable in frontends!
+     * @param params the parameter for this method
      * @return vector with Status entries.
      */
     virtual StatusEntries
-    status (const QString& path,
-            const bool descend = false,
-            const bool get_all = true,
-            const bool update = false,
-            const bool no_ignore = false,
-            const Revision revision = svn::Revision::HEAD,
-            bool detailed_remote = false,
-            const bool hide_externals = false) throw (ClientException);
+    status (const StatusParameter&params) throw (ClientException);
 
     /**
      * Returns the status of a single file in the path.
@@ -110,26 +103,17 @@ namespace svn
      * @param revision list specific revision when browsing remote, on working copies parameter will ignored
      * @return a Status with Statis.isVersioned = FALSE
      */
-    virtual Status
-    singleStatus (const QString& path,bool update=false,const Revision revision = svn::Revision::HEAD) throw (ClientException);
+    virtual StatusPtr
+    singleStatus (const Path& path,bool update=false,const Revision revision = svn::Revision::HEAD) throw (ClientException);
 
-  /**
+    /**
      * Executes a revision checkout.
-     * @param moduleName name of the module to checkout.
-     * @param destPath destination directory for checkout.
-     * @param revision the revision number to checkout. If the number is -1
-     *                 then it will checkout the latest revision.
-     * @param peg Revision to look up
-     * @param recurse whether you want it to checkout files recursively.
-     * @param ignore_externals if true don't process externals definitions.
+     * @param params the parameters to use
+     * @return revision number checked out
      * @exception ClientException
      */
-    virtual svn_revnum_t
-    checkout (const QString& moduleName, const Path & destPath,
-              const Revision & revision,
-              const Revision & peg = Revision::UNDEFINED,
-              bool recurse = true,
-              bool ignore_externals=false) throw (ClientException);
+    virtual Revision
+    checkout (const CheckoutParameter&params) throw (ClientException);
 
     /**
      * relocate wc @a from to @a to
@@ -139,13 +123,6 @@ namespace svn
     relocate (const Path & path, const QString &from_url,
               const QString &to_url, bool recurse) throw (ClientException);
 
-    /**
-     * Sets a single file for deletion.
-     * @exception ClientException
-     */
-    virtual void
-    remove (const Path & path, bool force) throw (ClientException);
-
     /**
      * Sets files for deletion.
      *
@@ -153,28 +130,34 @@ namespace svn
      * @param force force if files are locally modified
      * @exception ClientException
      */
-    virtual void
+    virtual svn::Revision
     remove (const Targets & targets,
-            bool force) throw (ClientException);
+            bool force,
+            bool keep_local = true,
+            const PropertiesMap&revProps = PropertiesMap()) throw (ClientException);
 
     /**
      * Reverts a couple of files to a pristiner state.
      * @exception ClientException
      */
     virtual void
-    revert (const Targets & targets, bool recurse) throw (ClientException);
+            revert (const Targets & targets,
+                    Depth depth,
+                    const StringArray&changelist=StringArray()
+                   ) throw (ClientException);
 
 
     /**
      * Adds a file to the repository.
      * @param path the path to add
-     * @param recurse if @a path is a folder add items recursive
+     * @param depth if @a path is a folder add items recursive depending on value if it. Pre-subversion 1.5 DepthInfinity is mapped to recursive, all other to not-recursive.
      * @param force if true, do not error on already-versioned items.
-     * @param no_ignore if false don't add files or directories that match ignore patterns. @since subversion 1.3
+     * @param no_ignore if false don't add files or directories that match ignore patterns.
+     * @param add_parents if true, go up to the next versioned folder and add all between path and this folder. Used only with subversion 1.5 or newer
      * @exception ClientException
+     * @sa svn::Depth
      */
-    virtual void
-    add (const Path & path, bool recurse,bool force=false, bool no_ignore=false) throw (ClientException);
+    virtual void add (const Path & path, svn::Depth depth,bool force=false, bool no_ignore=false, bool add_parents = true) throw (ClientException);
 
     /**
      * Updates the file or directory.
@@ -182,13 +165,15 @@ namespace svn
      * @param revision the revision number to checkout.
      *                 Revision::HEAD will checkout the
      *                 latest revision.
-     * @param recurse recursively update.
-     * @param ignore_externals ignore externals (only when Subversion 1.2 or above)
+     * @param depth Depthness for operation
+     * @param ignore_externals ignore externals
+     * @param allow_unversioned will operation not fail if there are unversioned items in tree with same name.
      * @exception ClientException
      */
     virtual Revisions
     update (const Targets & path, const Revision & revision,
-            bool recurse,bool ignore_externals) throw (ClientException);
+            Depth depth,bool ignore_externals,bool allow_unversioned,
+            bool sticky_depth) throw (ClientException);
 
     /**
      * Retrieves the contents for a specific @a revision of
@@ -203,7 +188,22 @@ namespace svn
     virtual QByteArray
     cat (const Path & path,
           const Revision & revision,
-          const Revision & peg_revision=svn_opt_revision_unspecified) throw (ClientException);
+          const Revision & peg_revision=Revision::UNDEFINED) throw (ClientException);
+    /**
+     * Retrieves the contents for a specific @a revision of
+     * a @a path at @a peg_revision
+     *
+     * @param buffer Stream to store content direct
+     * @param path path of file or directory
+     * @param peg_revision revision to base the URL
+     * @param revision revision to retrieve
+     * @exception ClientException
+     */
+    virtual void
+    cat(svn::stream::SvnStream&buffer,
+            const Path & path,
+            const Revision & revision,
+            const Revision & peg_revision) throw (ClientException);
 
     /**
      * Retrieves the contents for a specific @a revision of
@@ -219,7 +219,7 @@ namespace svn
     get (const Path & path,
           const QString  & target,
           const Revision & revision,
-          const Revision & peg_revision=svn_opt_revision_unspecified) throw (ClientException);
+          const Revision & peg_revision=Revision::UNDEFINED) throw (ClientException);
 
     /**
      * Retrieves the contents for a specific @a revision of
@@ -232,59 +232,44 @@ namespace svn
      * @param peg indicates in which revision path is valid
      */
     virtual void
-    annotate (AnnotatedFile&target,
-              const Path & path,
-              const Revision & revisionStart,
-              const Revision & revisionEnd,
-              const Revision & peg = Revision::UNDEFINED) throw (ClientException);
+    annotate (AnnotatedFile&target,const AnnotateParameter&params) throw (ClientException);
 
     /**
      * Commits changes to the repository. This usually requires
      * authentication, see Auth.
-     * @return Returns a long representing the revision. It returns a
-     *         -1 if the revision number is invalid.
-     * @param targets files to commit.
-     * @param message log message.
-     * @param recurse whether the operation should be done recursively.
-     * @param keep_locks if false unlock items in paths
+     * @param parameters CommitParameter to use
+     * @return Returns revision transferred or svn::Revision::UNDEFINED if the revision number is invalid.
      * @exception ClientException
      */
-    virtual svn_revnum_t
-    commit (const Targets & targets,
-            const QString& message,
-            bool recurse,bool keep_locks=true) throw (ClientException);
+    virtual svn::Revision
+    commit (const CommitParameter&parameters) throw (ClientException);
 
     /**
      * Copies a versioned file with the history preserved.
      * @exception ClientException
      */
-    virtual void
+    virtual svn::Revision
     copy (const Path & srcPath,
           const Revision & srcRevision,
           const Path & destPath) throw (ClientException);
+    /**
+     * Copies a versioned file with the history preserved.
+     * @since subversion 1.5 api
+     * @param parameter Class holding old required parameter
+     * @see svn_client_copy4,svn_client_copy5
+     * @exception ClientException
+     */
+    virtual svn::Revision
+    copy (const CopyParameter&parameter) throw (ClientException);
 
     /**
      * Moves or renames a file.
+     * @param parameter Class holding old required parameter
      * @exception ClientException
      */
-    virtual void
-    move (const Path & srcPath,
-          const Path & destPath,
-          bool force) throw (ClientException);
+    virtual svn::Revision
+    move (const CopyParameter&parameter) throw (ClientException);
 
-    /**
-     * Creates a directory directly in a repository or creates a
-     * directory on disk and schedules it for addition. If <i>path</i>
-     * is a URL then authentication is usually required, see Auth and
-     * the callback asks for a logmessage.
-     *
-     * @param path
-     * @param message log message. if it is QString::null asks when working on repository
-     * @exception ClientException
-     */
-    virtual void
-    mkdir (const Path & path,
-           const QString& message) throw (ClientException);
     /**
      * Creates a directory directly in a repository or creates a
      * directory on disk and schedules it for addition. If <i>path</i>
@@ -295,9 +280,12 @@ namespace svn
      * @param message log message. if it is QString::null asks when working on repository
      * @exception ClientException
      */
-    virtual void
+    virtual svn::Revision
     mkdir (const Targets & targets,
-           const QString& message) throw (ClientException);
+           const QString& message,
+           bool makeParent=true,
+           const PropertiesMap&revProps=PropertiesMap()
+          ) throw (ClientException);
 
     /**
      * Recursively cleans up a local directory, finishing any
@@ -312,42 +300,34 @@ namespace svn
      * Removes the 'conflicted' state on a file.
      * @exception ClientException
      */
-    virtual void
-    resolved (const Path & path, bool recurse) throw (ClientException);
+    virtual void resolve (const Path & path,Depth depth,const ConflictResult&resolution=ConflictResult()) throw (ClientException);
 
     /**
      * Exports the contents of either a subversion repository into a
      * 'clean' directory (meaning a directory with no administrative
      * directories).
      * @exception ClientException
-     * @param srcPath source path
-     * @param destPath a destination path that must not already exist.
-     * @param revision revision to use for the export
-     * @param peg the revision where the path is first looked up when exporting from a repository.
-     * @param overwrite overwrite existing files
-     * @param native_eol Either "LF", "CR" or "CRLF" or NULL.
-     * @param ignore_externals don't process externals definitions as part of this operation.
-     * @param recurse if true, export recursively. Otherwise, export just the directory represented by from and its immediate non-directory children.
+     * @param params Parameter to use
+     * @return revision exported
      */
-    virtual svn_revnum_t
-    doExport (const Path & srcPath,
-              const Path & destPath,
-              const Revision & revision,
-              const Revision & peg = Revision::UNDEFINED,
-              bool overwrite=false,
-              const QString&native_eol=QString::null,
-              bool ignore_externals = false,
-              bool recurse = true) throw (ClientException);
+    virtual Revision
+    doExport (const CheckoutParameter&params) throw (ClientException);
 
     /**
      * Update local copy to mirror a new url. This excapsulates the
      * svn_client_switch() client method.
      * @exception ClientException
      */
-    virtual svn_revnum_t
-    doSwitch (const Path & path, const QString& url,
+    virtual Revision
+    doSwitch (
+              const Path & path, const QString& url,
               const Revision & revision,
-              bool recurse) throw (ClientException);
+              Depth depth,
+              const Revision & peg=Revision::UNDEFINED,
+              bool sticky_depth = true,
+              bool ignore_externals=false,
+              bool allow_unversioned=false
+             ) throw (ClientException);
 
     /**
      * Import file or directory PATH into repository directory URL at
@@ -355,25 +335,28 @@ namespace svn
      * @param path path to import
      * @param url
      * @param message log message.
-     * @param recurse
+     * @param depth kind of recurse operation
+     * @param no_ignore if false, don't add items matching global ignore pattern
+     * @param no_unknown_nodetype if true ignore files type not known like pipes or device files
      * @exception ClientException
      */
-    virtual void
-    import (const Path & path, const QString& url,
-            const QString& message,
-            bool recurse) throw (ClientException);
+    virtual svn::Revision
+            import (const Path & path, const QString& url,
+                    const QString& message,
+                    svn::Depth depth,
+                    bool no_ignore,bool no_unknown_nodetype,
+                    const PropertiesMap&revProps=PropertiesMap()) throw (ClientException);
 
     /**
      * Merge changes from two paths into a new local path.
      * @exception ClientException
+     * @sa svn::MergeParameter
      */
     virtual void
-    merge (const Path & path1, const Revision & revision1,
-           const Path & path2, const Revision & revision2,
-           const Path & localPath, bool force,
-           bool recurse,
-           bool notice_ancestry=false,
-           bool dry_run=false) throw (ClientException);
+    merge (const MergeParameter&parameters) throw (ClientException);
+
+    virtual void
+    merge_peg(const MergeParameter&parameters) throw (ClientException);
 
     /**
      * Retrieve information for the given path
@@ -388,31 +371,13 @@ namespace svn
      * @since subversion 1.2
      */
     virtual InfoEntries
-    info(const QString &path,
-          bool rec,
-          const Revision & rev,
-          const Revision & peg_revision=svn_opt_revision_unspecified) throw (ClientException);
-    /**
-     * Retrieve log information for the given path
-     * Loads the log messages result set. The first
-     * entry  is the youngest revision.
-     *
-     * You can use the constants Revision::START and
-     * Revision::HEAD
-     *
-     * @param path
-     * @param revisionStart
-     * @param revisionEnd
-     * @param discoverChangedPaths
-     * @param strictNodeHistory
-     * @param limit (ignored when subversion 1.1 API)
-     * @return a vector with log entries
-     */
-    virtual const LogEntries *
-    log (const QString& path, const Revision & revisionStart,
-         const Revision & revisionEnd,
-         bool discoverChangedPaths=false,
-         bool strictNodeHistory=true,int limit = 0) throw (ClientException);
+    info(const Path &path,
+         Depth depth,
+         const Revision & rev,
+         const Revision & peg_revision=Revision::UNDEFINED,
+         const StringArray&changelists=StringArray()
+        ) throw (ClientException);
+
     /**
      * Retrieve log information for the given path
      * Loads the log messages result set. Result will stored
@@ -421,21 +386,13 @@ namespace svn
      * You can use the constants Revision::START and
      * Revision::HEAD
      *
-     * @param path
-     * @param revisionStart
-     * @param revisionEnd
-     * @param target the logmap where to store the entries
-     * @param discoverChangedPaths
-     * @param strictNodeHistory
-     * @param limit (ignored when subversion 1.1 API)
+     * @param params Parameter to use for log
+     * @param target where to store the resulting logs
      * @return true if success
+     * @sa LogParameter
      */
     virtual bool
-    log (const QString& path, const Revision & revisionStart,
-         const Revision & revisionEnd,
-         LogEntriesMap&target,
-         bool discoverChangedPaths=false,
-         bool strictNodeHistory=true,int limit = 0) throw (ClientException);
+    log (const LogParameter&params,LogEntriesMap&target) throw (ClientException);
 
     /**
      * Produce diff output which describes the delta between
@@ -445,26 +402,14 @@ namespace svn
      * A ClientException will be thrown if either @a revision1 or
      * @a revision2 has an `unspecified' or unrecognized `kind'.
      *
-     * @param tmpPath prefix for a temporary directory needed by diff.
-     * Filenames will have ".tmp" and similar added to this prefix in
-     * order to ensure uniqueness.
-     * @param path path of the file.
-     * @param revision1 one of the revisions to check.
-     * @param revision2 the other revision.
-     * @param recurse whether the operation should be done recursively.
-     * @param ignoreAncestry whether the files will be checked for
-     * relatedness.
-     * @param noDiffDeleted if true, no diff output will be generated
-     * on deleted files.
-     * @param ignore_contenttype if true generate diff even the items are marked as binaries
+     * @param options set of options required for diff
      * @return delta between the files
      * @exception ClientException
      */
     virtual QByteArray
-    diff (const Path & tmpPath, const Path & path,
-          const Revision & revision1, const Revision & revision2,
-          const bool recurse, const bool ignoreAncestry,
-          const bool noDiffDeleted,const bool ignore_contenttype) throw (ClientException);
+            diff_peg (const DiffParameter&options)
+            throw (ClientException);
+
     /**
      * Produce diff output which describes the delta between
      * @a path1/@a revision1 and @a path2/@a revision2. @a path2
@@ -473,26 +418,13 @@ namespace svn
      * A ClientException will be thrown if either @a revision1 or
      * @a revision2 has an `unspecified' or unrecognized `kind'.
      *
-     * @param tmpPath prefix for a temporary directory needed by diff.
-     * Filenames will have ".tmp" and similar added to this prefix in
-     * order to ensure uniqueness.
-     * @param path1 first file or folder to diff.
-     * @param path2 second file or folder to diff.
-     * @param revision1 one of the revisions to check (path1).
-     * @param revision2 the other revision (path2).
-     * @param recurse whether the operation should be done recursively.
-     * @param ignoreAncestry whether the files will be checked for
-     * relatedness.
-     * @param noDiffDeleted if true, no diff output will be generated on deleted files.
-     * @param ignore_contenttype if true generate diff even the items are marked as binaries
+     * @param options set of options required for diff
      * @return delta between the files
      * @exception ClientException
      */
     virtual QByteArray
-    diff (const Path & tmpPath, const Path & path1,const Path & path2,
-          const Revision & revision1, const Revision & revision2,
-          const bool recurse, const bool ignoreAncestry,
-          const bool noDiffDeleted,const bool ignore_contenttype) throw (ClientException);
+            diff (const DiffParameter&options)
+            throw (ClientException);
 
     /**
      * lists entries in @a pathOrUrl no matter whether local or
@@ -501,16 +433,16 @@ namespace svn
      * @param pathOrUrl
      * @param revision
      * @param peg at wich revision path exists
-     * @param recurse
+     * @param depth @sa svn::Depth
      * @param retrieve_locks check for REPOSITORY locks while listing
      * @return a vector of directory entries, each with
      *         a relative path (only filename)
      */
     virtual DirEntries
-    list (const QString& pathOrUrl,
+    list (const Path& pathOrUrl,
           const Revision& revision,
           const Revision& peg,
-          bool recurse,bool retrieve_locks) throw (ClientException);
+          svn::Depth depth,bool retrieve_locks) throw (ClientException);
 
     /**
      * lists properties in @a path no matter whether local or
@@ -522,11 +454,12 @@ namespace svn
      * @param recurse
      * @return PropertiesList
      */
-    virtual PathPropertiesMapList
+    virtual PathPropertiesMapListPtr
     proplist(const Path &path,
              const Revision &revision,
              const Revision &peg,
-             bool recurse=false);
+             Depth depth=DepthEmpty,
+             const StringArray&changelists=StringArray());
 
     /**
      * lists one property in @a path no matter whether local or
@@ -539,48 +472,32 @@ namespace svn
      * @param recurse
      * @return PathPropertiesMapList
      */
-    virtual PathPropertiesMapList
+    virtual QPair<QLONG,PathPropertiesMapList>
     propget(const QString& propName,
             const Path &path,
             const Revision &revision,
             const Revision &peg,
-            bool recurse=false);
+            Depth depth = svn::DepthEmpty,
+            const StringArray&changelists=StringArray());
 
     /**
      * set property in @a path no matter whether local or
      * repository
      *
-     * @param path
-     * @param revision
-     * @param propName
-     * @param propValue
-     * @param recurse
-     * @param skip_check if true skip validity checks
-     * @return PropertiesList
+     * @param params svn::PropertiesParamter holding required values.
+     * Following is used:<br>
+     * <ul>
+     * <li> svn::PropertiesParameter::propertyName()
+     * <li> svn::PropertiesParameter::propertyValue()
+     * <li> svn::PropertiesParameter::depth()
+     * <li> svn::PropertiesParameter::skipCheck()
+     * <li> svn::PropertiesParameter::revision()
+     * <li> svn::PropertiesParameter::changeList()
+     * <li> svn::PropertiesParameter::revisionProperties()
+     * </ul>
      */
     virtual void
-    propset(const QString& propName,
-            const QString& propValue,
-            const Path &path,
-            const Revision &revision,
-            bool recurse=false,
-            bool skip_check=false);
-
-    /**
-     * delete property in @a path no matter whether local or
-     * repository
-     *
-     * @param propName
-     * @param path
-     * @param revision
-     * @param recurse
-     */
-    virtual void
-    propdel(const QString& propName,
-            const Path &path,
-            const Revision &revision,
-            bool recurse=false);
-
+    propset(const PropertiesParameter&params);
 
     /**
      * lists revision properties in @a path no matter whether local or
@@ -590,7 +507,7 @@ namespace svn
      * @param revision
      * @return PropertiesList
      */
-    virtual QPair<svn_revnum_t,PropertiesMap>
+    virtual QPair<QLONG,PropertiesMap>
     revproplist(const Path &path,
                 const Revision &revision);
 
@@ -603,7 +520,7 @@ namespace svn
      * @param revision
      * @return PropertiesList
      */
-    QPair<svn_revnum_t,QString>
+    QPair<QLONG,QString>
     revpropget(const QString& propName,
                const Path &path,
                const Revision &revision);
@@ -612,19 +529,12 @@ namespace svn
      * set revision property in @a path no matter whether local or
      * repository
      *
-     * @param propName
-     * @param propValue
-     * @param path
-     * @param revision
-     * @param force
+     * @param params parameter to use
      * @return Revision
+     * @sa PropertiesParameter
      */
-    virtual svn_revnum_t
-    revpropset(const QString& propName,
-               const QString& propValue,
-               const Path &path,
-               const Revision &revision,
-               bool force=false);
+    virtual QLONG
+    revpropset(const PropertiesParameter&params);
 
     /**
      * delete revision property in @a path no matter whether local or
@@ -636,11 +546,10 @@ namespace svn
      * @param force
      * @return Revision
      */
-    virtual svn_revnum_t
+    virtual QLONG
     revpropdel(const QString& propName,
                const Path &path,
-               const Revision &revision,
-               bool force=false);
+               const Revision &revision);
 
   /**
    * lock files in repository or working copy
@@ -669,13 +578,30 @@ namespace svn
     url2Revision(const QString&revstring,
             Revision&start);
 
+    virtual bool RepoHasCapability(const Path&repository,Capability capability);
+
     struct sBaton {
+        sBaton():m_context(0),m_data(0),m_revstack(0),excludeList(0){}
         Context*m_context;
         void*m_data;
+        void*m_revstack;
+        const StringArray*excludeList;
     };
 
+    struct propBaton {
+        Context*m_context;
+        PathPropertiesMapList*resultlist;
+    };
+    static void checkErrorThrow(svn_error_t*error)throw(ClientException)
+    {
+        if (!error || error->apr_err==APR_SUCCESS) {
+            return;
+        }
+        throw ClientException (error);
+    }
+
   private:
-    Context * m_context;
+    ContextP m_context;
 
     /**
      * disallow assignment operator
@@ -688,12 +614,12 @@ namespace svn
     Client_impl (const Client &);
 
     DirEntries
-    list_simple(const QString& pathOrUrl,
+    list_simple(const Path& pathOrUrl,
           const Revision& revision,
           const Revision& peg,
           bool recurse) throw (ClientException);
     DirEntries
-    list_locks(const QString& pathOrUrl,
+    list_locks(const Path& pathOrUrl,
           const Revision& revision,
           const Revision& peg,
           bool recurse) throw (ClientException);
@@ -702,7 +628,24 @@ namespace svn
                 const Revision & revision,
                 const Revision & peg_revision,
                 svn::stream::SvnStream&);
-  };
+
+    static apr_hash_t * map2hash(const PropertiesMap&,const Pool&);
+    static apr_array_header_t * revListToHeader(const RevisionRanges&,const Pool&);
+
+    //! helper method
+    virtual void
+    merge_peg(const Path&src,
+              const RevisionRange&range,
+              const Revision&peg,
+              const Path&targetWc,
+              Depth depth,
+              bool notice_ancestry,
+              bool dry_run,
+              bool force,
+              const StringArray&merge_options
+            ) throw (ClientException);
+
+    };
 
 }
 
